package java.io;

import java.util.Arrays;

public class CharArrayWriter extends Writer
{
    protected char[] buf;
    protected int count;
    
    public CharArrayWriter() {
        this(32);
    }
    
    public CharArrayWriter(int i) {
        if (i < 0) {
            throw new IllegalArgumentException("Negative initial size: " + i);
        }
        this.buf = new char[i];
        return;
    }
    
    @Override
    public void write(int n) {
        synchronized (this.lock) {
            int n2 = this.count + 1;
            if (n2 > this.buf.length) {
                this.buf = Arrays.copyOf(this.buf, Math.max(this.buf.length << 1, n2));
            }
            this.buf[this.count] = (char)n;
            this.count = n2;
        }
    }
    
    @Override
    public void write(char[] array, int n, int n2) {
        if (n < 0 || n > array.length || n2 < 0 || n + n2 > array.length || n + n2 < 0) {
            throw new IndexOutOfBoundsException();
        }
        if (n2 == 0) {
            return;
        }
        synchronized (this.lock) {
            int n3 = this.count + n2;
            if (n3 > this.buf.length) {
                this.buf = Arrays.copyOf(this.buf, Math.max(this.buf.length << 1, n3));
            }
            System.arraycopy(array, n, this.buf, this.count, n2);
            this.count = n3;
        }
        return;
    }
    
    @Override
    public void write(String s, int srcBegin, int n) {
        synchronized (this.lock) {
            int n2 = this.count + n;
            if (n2 > this.buf.length) {
                this.buf = Arrays.copyOf(this.buf, Math.max(this.buf.length << 1, n2));
            }
            s.getChars(srcBegin, srcBegin + n, this.buf, this.count);
            this.count = n2;
        }
    }
    
    public void writeTo(Writer writer) throws IOException {
        synchronized (this.lock) {
            writer.write(this.buf, 0, this.count);
        }
    }
    
    @Override
    public CharArrayWriter append(CharSequence charSequence) {
        String s = (charSequence == null) ? "null" : charSequence.toString();
        this.write(s, 0, s.length());
        return this;
    }
    
    @Override
    public CharArrayWriter append(CharSequence charSequence, int n, int n2) {
        String string = ((charSequence == null) ? "null" : charSequence).subSequence(n, n2).toString();
        this.write(string, 0, string.length());
        return this;
    }
    
    @Override
    public CharArrayWriter append(char c) {
        this.write((int)c);
        return this;
    }
    
    public void reset() {
        this.count = 0;
    }
    
    public char[] toCharArray() {
        synchronized (this.lock) {
            return Arrays.copyOf(this.buf, this.count);
        }
    }
    
    public int size() {
        return this.count;
    }
    
    @Override
    public String toString() {
        synchronized (this.lock) {
            return new String(this.buf, 0, this.count);
        }
    }
    
    @Override
    public void flush() {
    }
    
    @Override
    public void close() {
    }
}
